@@ -19,12 +19,10 @@
 
 #include "config.h"
 
-#include "gdkdevice.h"
-
 #include "gdkdeviceprivate.h"
-#include "gdkintl.h"
+#include "gdkdisplayprivate.h"
 #include "gdkinternals.h"
-
+#include "gdkintl.h"
 
 typedef struct _GdkAxisInfo GdkAxisInfo;
 
@@ -41,15 +39,14 @@ struct _GdkAxisInfo
   gdouble resolution;
 };
 
-struct _GdkDevicePrivate
-{
-  GdkDeviceManager *device_manager;
-  GdkDisplay *display;
-  GdkDevice *associated;
-  GdkDeviceType type;
-  GArray *axes;
+enum {
+  CHANGED,
+  LAST_SIGNAL
 };
 
+static guint signals [LAST_SIGNAL] = { 0 };
+
+
 static void gdk_device_dispose      (GObject      *object);
 static void gdk_device_set_property (GObject      *object,
                                      guint         prop_id,
@@ -94,8 +91,8 @@ gdk_device_class_init (GdkDeviceClass *klass)
    * Since: 3.0
    */
   g_object_class_install_property (object_class,
-				   PROP_DISPLAY,
-				   g_param_spec_object ("display",
+                                   PROP_DISPLAY,
+                                   g_param_spec_object ("display",
                                                         P_("Device Display"),
                                                         P_("Display which the device belongs to"),
                                                         GDK_TYPE_DISPLAY,
@@ -109,8 +106,8 @@ gdk_device_class_init (GdkDeviceClass *klass)
    * Since: 3.0
    */
   g_object_class_install_property (object_class,
-				   PROP_DEVICE_MANAGER,
-				   g_param_spec_object ("device-manager",
+                                   PROP_DEVICE_MANAGER,
+                                   g_param_spec_object ("device-manager",
                                                         P_("Device manager"),
                                                         P_("Device manager which the device belongs to"),
                                                         GDK_TYPE_DEVICE_MANAGER,
@@ -124,8 +121,8 @@ gdk_device_class_init (GdkDeviceClass *klass)
    * Since: 3.0
    */
   g_object_class_install_property (object_class,
-				   PROP_NAME,
-				   g_param_spec_string ("name",
+                                   PROP_NAME,
+                                   g_param_spec_string ("name",
                                                         P_("Device name"),
                                                         P_("Device name"),
                                                         NULL,
@@ -156,8 +153,8 @@ gdk_device_class_init (GdkDeviceClass *klass)
    * Since: 3.0
    */
   g_object_class_install_property (object_class,
-				   PROP_ASSOCIATED_DEVICE,
-				   g_param_spec_object ("associated-device",
+                                   PROP_ASSOCIATED_DEVICE,
+                                   g_param_spec_object ("associated-device",
                                                         P_("Associated device"),
                                                         P_("Associated pointer or keyboard with this device"),
                                                         GDK_TYPE_DEVICE,
@@ -170,8 +167,8 @@ gdk_device_class_init (GdkDeviceClass *klass)
    * Since: 3.0
    */
   g_object_class_install_property (object_class,
-				   PROP_INPUT_SOURCE,
-				   g_param_spec_enum ("input-source",
+                                   PROP_INPUT_SOURCE,
+                                   g_param_spec_enum ("input-source",
                                                       P_("Input source"),
                                                       P_("Source type for the device"),
                                                       GDK_TYPE_INPUT_SOURCE,
@@ -187,7 +184,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
    */
   g_object_class_install_property (object_class,
                                    PROP_INPUT_MODE,
-				   g_param_spec_enum ("input-mode",
+                                   g_param_spec_enum ("input-mode",
                                                       P_("Input mode for the device"),
                                                       P_("Input mode for the device"),
                                                       GDK_TYPE_INPUT_MODE,
@@ -202,8 +199,8 @@ gdk_device_class_init (GdkDeviceClass *klass)
    * Since: 3.0
    */
   g_object_class_install_property (object_class,
-				   PROP_HAS_CURSOR,
-				   g_param_spec_boolean ("has-cursor",
+                                   PROP_HAS_CURSOR,
+                                   g_param_spec_boolean ("has-cursor",
                                                          P_("Whether the device has a cursor"),
                                                          P_("Whether there is a visible cursor following device motion"),
                                                          FALSE,
@@ -217,57 +214,66 @@ gdk_device_class_init (GdkDeviceClass *klass)
    * Since: 3.0
    */
   g_object_class_install_property (object_class,
-				   PROP_N_AXES,
-				   g_param_spec_uint ("n-axes",
+                                   PROP_N_AXES,
+                                   g_param_spec_uint ("n-axes",
                                                       P_("Number of axes in the device"),
                                                       P_("Number of axes in the device"),
                                                       0, G_MAXUINT, 0,
                                                       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
-  g_type_class_add_private (object_class, sizeof (GdkDevicePrivate));
+  /**
+   * GdkDevice::changed:
+   * @device: the #GdkDevice that changed.
+   *
+   * The ::changed signal is emitted either when the #GdkDevice
+   * has changed the number of either axes or keys. For example
+   * In X this will normally happen when the slave device routing
+   * events through the master device changes (for example, user
+   * switches from the USB mouse to a tablet), in that case the
+   * master device will change to reflect the new slave device
+   * axes and keys.
+   */
+  signals[CHANGED] =
+    g_signal_new (g_intern_static_string ("changed"),
+                  G_TYPE_FROM_CLASS (object_class),
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
 }
 
 static void
 gdk_device_init (GdkDevice *device)
 {
-  GdkDevicePrivate *priv;
-
-  device->priv = priv = G_TYPE_INSTANCE_GET_PRIVATE (device,
-                                                     GDK_TYPE_DEVICE,
-                                                     GdkDevicePrivate);
-
-  priv->axes = g_array_new (FALSE, TRUE, sizeof (GdkAxisInfo));
+  device->axes = g_array_new (FALSE, TRUE, sizeof (GdkAxisInfo));
 }
 
 static void
 gdk_device_dispose (GObject *object)
 {
-  GdkDevicePrivate *priv;
-  GdkDevice *device;
+  GdkDevice *device = GDK_DEVICE (object);
 
-  device = GDK_DEVICE (object);
-  priv = device->priv;
+  if (device->type == GDK_DEVICE_TYPE_SLAVE)
+    _gdk_device_remove_slave (device->associated, device);
 
-  if (priv->associated)
+  if (device->associated)
     {
-      _gdk_device_set_associated_device (priv->associated, NULL);
-      g_object_unref (priv->associated);
-      priv->associated = NULL;
+      _gdk_device_set_associated_device (device->associated, NULL);
+      g_object_unref (device->associated);
+      device->associated = NULL;
     }
 
-  if (priv->axes)
+  if (device->axes)
     {
-      g_array_free (priv->axes, TRUE);
-      priv->axes = NULL;
+      g_array_free (device->axes, TRUE);
+      device->axes = NULL;
     }
 
   g_free (device->name);
   g_free (device->keys);
-  g_free (device->axes);
 
   device->name = NULL;
   device->keys = NULL;
-  device->axes = NULL;
 
   G_OBJECT_CLASS (gdk_device_parent_class)->dispose (object);
 }
@@ -279,15 +285,14 @@ gdk_device_set_property (GObject      *object,
                          GParamSpec   *pspec)
 {
   GdkDevice *device = GDK_DEVICE (object);
-  GdkDevicePrivate *priv = device->priv;
 
   switch (prop_id)
     {
     case PROP_DISPLAY:
-      priv->display = g_value_get_object (value);
+      device->display = g_value_get_object (value);
       break;
     case PROP_DEVICE_MANAGER:
-      priv->device_manager = g_value_get_object (value);
+      device->manager = g_value_get_object (value);
       break;
     case PROP_NAME:
       if (device->name)
@@ -296,7 +301,7 @@ gdk_device_set_property (GObject      *object,
       device->name = g_value_dup_string (value);
       break;
     case PROP_TYPE:
-      priv->type = g_value_get_enum (value);
+      device->type = g_value_get_enum (value);
       break;
     case PROP_INPUT_SOURCE:
       device->source = g_value_get_enum (value);
@@ -320,25 +325,23 @@ gdk_device_get_property (GObject    *object,
                          GParamSpec *pspec)
 {
   GdkDevice *device = GDK_DEVICE (object);
-  GdkDevicePrivate *priv = device->priv;
 
   switch (prop_id)
     {
     case PROP_DISPLAY:
-      g_value_set_object (value, priv->display);
+      g_value_set_object (value, device->display);
       break;
     case PROP_DEVICE_MANAGER:
-      g_value_set_object (value, priv->device_manager);
+      g_value_set_object (value, device->manager);
       break;
     case PROP_ASSOCIATED_DEVICE:
-      g_value_set_object (value, priv->associated);
+      g_value_set_object (value, device->associated);
       break;
     case PROP_NAME:
-      g_value_set_string (value,
-                          device->name);
+      g_value_set_string (value, device->name);
       break;
     case PROP_TYPE:
-      g_value_set_enum (value, priv->type);
+      g_value_set_enum (value, device->type);
       break;
     case PROP_INPUT_SOURCE:
       g_value_set_enum (value, device->source);
@@ -347,11 +350,10 @@ gdk_device_get_property (GObject    *object,
       g_value_set_enum (value, device->mode);
       break;
     case PROP_HAS_CURSOR:
-      g_value_set_boolean (value,
-                           device->has_cursor);
+      g_value_set_boolean (value, device->has_cursor);
       break;
     case PROP_N_AXES:
-      g_value_set_uint (value, priv->axes->len);
+      g_value_set_uint (value, device->axes->len);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -367,7 +369,7 @@ gdk_device_get_property (GObject    *object,
  * or %NULL.
  * @mask: location to store the modifiers, or %NULL.
  *
- * Gets the current state of a device relative to @window.
+ * Gets the current state of a pointer device relative to @window.
  */
 void
 gdk_device_get_state (GdkDevice       *device,
@@ -376,6 +378,7 @@ gdk_device_get_state (GdkDevice       *device,
                       GdkModifierType *mask)
 {
   g_return_if_fail (GDK_IS_DEVICE (device));
+  g_return_if_fail (gdk_device_get_source (device) != GDK_SOURCE_KEYBOARD);
   g_return_if_fail (GDK_IS_WINDOW (window));
 
   if (GDK_DEVICE_GET_CLASS (device)->get_state)
@@ -391,7 +394,7 @@ gdk_device_get_state (GdkDevice       *device,
  * @events: (array length=n_events) (out) (transfer none): location to store a newly-allocated array of #GdkTimeCoord, or %NULL
  * @n_events: location to store the length of @events, or %NULL
  *
- * Obtains the motion history for a device; given a starting and
+ * Obtains the motion history for a pointer device; given a starting and
  * ending timestamp, return all events in the motion history for
  * the device in the given range of time. Some windowing systems
  * do not support motion history, in which case, %FALSE will
@@ -410,6 +413,7 @@ gdk_device_get_history (GdkDevice      *device,
                         gint           *n_events)
 {
   g_return_val_if_fail (GDK_IS_DEVICE (device), FALSE);
+  g_return_val_if_fail (gdk_device_get_source (device) != GDK_SOURCE_KEYBOARD, FALSE);
   g_return_val_if_fail (GDK_IS_WINDOW (window), FALSE);
 
   if (n_events)
@@ -438,7 +442,7 @@ _gdk_device_allocate_history (GdkDevice *device,
 
   for (i = 0; i < n_events; i++)
     result[i] = g_malloc (sizeof (GdkTimeCoord) -
-			  sizeof (double) * (GDK_MAX_TIMECOORD_AXES - device->num_axes));
+                          sizeof (double) * (GDK_MAX_TIMECOORD_AXES - device->axes->len));
   return result;
 }
 
@@ -493,6 +497,7 @@ gboolean
 gdk_device_get_has_cursor (GdkDevice *device)
 {
   g_return_val_if_fail (GDK_IS_DEVICE (device), FALSE);
+  g_return_val_if_fail (gdk_device_get_source (device) != GDK_SOURCE_KEYBOARD, FALSE);
 
   return device->has_cursor;
 }
@@ -524,11 +529,12 @@ gdk_device_get_source (GdkDevice *device)
  **/
 void
 gdk_device_set_source (GdkDevice      *device,
-		       GdkInputSource  source)
+                       GdkInputSource  source)
 {
   g_return_if_fail (GDK_IS_DEVICE (device));
 
   device->source = source;
+  g_object_notify (G_OBJECT (device), "input-source");
 }
 
 /**
@@ -573,17 +579,30 @@ gdk_device_set_mode (GdkDevice    *device,
       gdk_device_get_device_type (device) == GDK_DEVICE_TYPE_MASTER)
     return FALSE;
 
-  /* FIXME: setting has_cursor when mode is window? */
-
   device->mode = mode;
   g_object_notify (G_OBJECT (device), "input-mode");
 
-  if (gdk_device_get_device_type (device) != GDK_DEVICE_TYPE_MASTER)
-    _gdk_input_check_extension_events (device);
-
   return TRUE;
 }
 
+/**
+ * gdk_device_get_n_keys:
+ * @device: a #GdkDevice
+ *
+ * Returns the number of keys the device currently has.
+ *
+ * Returns: the number of keys.
+ *
+ * Since: 2.24
+ **/
+gint
+gdk_device_get_n_keys (GdkDevice *device)
+{
+  g_return_val_if_fail (GDK_IS_DEVICE (device), 0);
+
+  return device->num_keys;
+}
+
 /**
  * gdk_device_get_key:
  * @device: a #GdkDevice.
@@ -632,9 +651,9 @@ gdk_device_get_key (GdkDevice       *device,
  **/
 void
 gdk_device_set_key (GdkDevice      *device,
-		    guint           index_,
-		    guint           keyval,
-		    GdkModifierType modifiers)
+                    guint           index_,
+                    guint           keyval,
+                    GdkModifierType modifiers)
 {
   g_return_if_fail (GDK_IS_DEVICE (device));
   g_return_if_fail (index_ < device->num_keys);
@@ -645,7 +664,7 @@ gdk_device_set_key (GdkDevice      *device,
 
 /**
  * gdk_device_get_axis_use:
- * @device: a #GdkDevice.
+ * @device: a pointer #GdkDevice.
  * @index_: the index of the axis.
  *
  * Returns the axis use for @index_.
@@ -658,15 +677,20 @@ GdkAxisUse
 gdk_device_get_axis_use (GdkDevice *device,
                          guint      index_)
 {
-  g_return_val_if_fail (GDK_IS_DEVICE (device), GDK_AXIS_IGNORE);
-  g_return_val_if_fail (index_ < device->num_axes, GDK_AXIS_IGNORE);
+  GdkAxisInfo *info;
 
-  return device->axes[index_].use;
+  g_return_val_if_fail (GDK_IS_DEVICE (device), GDK_AXIS_IGNORE);
+  g_return_val_if_fail (gdk_device_get_source (device) != GDK_SOURCE_KEYBOARD, GDK_AXIS_IGNORE);
+  g_return_val_if_fail (index_ < device->axes->len, GDK_AXIS_IGNORE);
+
+  info = &g_array_index (device->axes, GdkAxisInfo, index_);
+
+  return info->use;
 }
 
 /**
  * gdk_device_set_axis_use:
- * @device: a #GdkDevice
+ * @device: a pointer #GdkDevice
  * @index_: the index of the axis
  * @use: specifies how the axis is used
  *
@@ -674,36 +698,33 @@ gdk_device_get_axis_use (GdkDevice *device,
  **/
 void
 gdk_device_set_axis_use (GdkDevice   *device,
-			 guint        index_,
-			 GdkAxisUse   use)
+                         guint        index_,
+                         GdkAxisUse   use)
 {
-  GdkDevicePrivate *priv;
   GdkAxisInfo *info;
 
   g_return_if_fail (GDK_IS_DEVICE (device));
-  g_return_if_fail (index_ < device->num_axes);
+  g_return_if_fail (gdk_device_get_source (device) != GDK_SOURCE_KEYBOARD);
+  g_return_if_fail (index_ < device->axes->len);
 
-  priv = device->priv;
-  info = &g_array_index (priv->axes, GdkAxisInfo, index_);
+  info = &g_array_index (device->axes, GdkAxisInfo, index_);
   info->use = use;
 
-  device->axes[index_].use = use;
-
   switch (use)
     {
     case GDK_AXIS_X:
     case GDK_AXIS_Y:
-      device->axes[index_].min = info->min_axis = 0;
-      device->axes[index_].max = info->max_axis = 0;
+      info->min_axis = 0;
+      info->max_axis = 0;
       break;
     case GDK_AXIS_XTILT:
     case GDK_AXIS_YTILT:
-      device->axes[index_].min = info->min_axis = -1;
-      device->axes[index_].max = info->max_axis = 1;
+      info->min_axis = -1;
+      info->max_axis = 1;
       break;
     default:
-      device->axes[index_].min = info->min_axis = 0;
-      device->axes[index_].max = info->max_axis = 1;
+      info->min_axis = 0;
+      info->max_axis = 1;
       break;
     }
 }
@@ -714,21 +735,17 @@ gdk_device_set_axis_use (GdkDevice   *device,
  *
  * Returns the #GdkDisplay to which @device pertains.
  *
- * Returns: a #GdkDisplay. This memory is owned by GTK+,
- *          and must not be freed or unreffed.
+ * Returns: (transfer none): a #GdkDisplay. This memory is owned
+ *          by GTK+, and must not be freed or unreffed.
  *
  * Since: 3.0
  **/
 GdkDisplay *
 gdk_device_get_display (GdkDevice *device)
 {
-  GdkDevicePrivate *priv;
-
   g_return_val_if_fail (GDK_IS_DEVICE (device), NULL);
 
-  priv = device->priv;
-
-  return priv->display;
+  return device->display;
 }
 
 /**
@@ -745,44 +762,105 @@ gdk_device_get_display (GdkDevice *device)
  * If @device is of type %GDK_DEVICE_TYPE_FLOATING, %NULL will be
  * returned, as there is no associated device.
  *
- * Returns: The associated device, or %NULL
+ * Returns: (transfer none): The associated device, or %NULL
  *
  * Since: 3.0
  **/
 GdkDevice *
 gdk_device_get_associated_device (GdkDevice *device)
 {
-  GdkDevicePrivate *priv;
-
   g_return_val_if_fail (GDK_IS_DEVICE (device), NULL);
 
-  priv = device->priv;
+  return device->associated;
+}
 
-  return priv->associated;
+static void
+_gdk_device_set_device_type (GdkDevice     *device,
+                             GdkDeviceType  type)
+{
+  if (device->type != type)
+    {
+      device->type = type;
+
+      g_object_notify (G_OBJECT (device), "type");
+    }
 }
 
 void
 _gdk_device_set_associated_device (GdkDevice *device,
                                    GdkDevice *associated)
 {
-  GdkDevicePrivate *priv;
-
   g_return_if_fail (GDK_IS_DEVICE (device));
   g_return_if_fail (associated == NULL || GDK_IS_DEVICE (associated));
 
-  priv = device->priv;
-
-  if (priv->associated == associated)
+  if (device->associated == associated)
     return;
 
-  if (priv->associated)
+  if (device->associated)
     {
-      g_object_unref (priv->associated);
-      priv->associated = NULL;
+      g_object_unref (device->associated);
+      device->associated = NULL;
     }
 
   if (associated)
-    priv->associated = g_object_ref (associated);
+    device->associated = g_object_ref (associated);
+
+  if (device->type != GDK_DEVICE_TYPE_MASTER)
+    {
+      if (device->associated)
+        _gdk_device_set_device_type (device, GDK_DEVICE_TYPE_SLAVE);
+      else
+        _gdk_device_set_device_type (device, GDK_DEVICE_TYPE_FLOATING);
+    }
+}
+
+/**
+ * gdk_device_list_slave_devices:
+ * @device: a #GdkDevice
+ *
+ * If the device if of type %GDK_DEVICE_TYPE_MASTER, it will return
+ * the list of slave devices attached to it, otherwise it will return
+ * %NULL
+ *
+ * Returns: (transfer container): the list of slave devices, or %NULL. The
+ *          list must be freed with g_list_free(), the contents of the list
+ *          are owned by GTK+ and should not be freed.
+ **/
+GList *
+gdk_device_list_slave_devices (GdkDevice *device)
+{
+  g_return_val_if_fail (GDK_IS_DEVICE (device), NULL);
+  g_return_val_if_fail (gdk_device_get_device_type (device) != GDK_DEVICE_TYPE_MASTER, NULL);
+
+  return g_list_copy (device->slaves);
+}
+
+void
+_gdk_device_add_slave (GdkDevice *device,
+                       GdkDevice *slave)
+{
+  g_return_if_fail (gdk_device_get_device_type (device) == GDK_DEVICE_TYPE_MASTER);
+  g_return_if_fail (gdk_device_get_device_type (slave) != GDK_DEVICE_TYPE_MASTER);
+
+  if (!g_list_find (device->slaves, slave))
+    device->slaves = g_list_prepend (device->slaves, slave);
+}
+
+void
+_gdk_device_remove_slave (GdkDevice *device,
+                          GdkDevice *slave)
+{
+  GList *elem;
+
+  g_return_if_fail (gdk_device_get_device_type (device) == GDK_DEVICE_TYPE_MASTER);
+  g_return_if_fail (gdk_device_get_device_type (slave) != GDK_DEVICE_TYPE_MASTER);
+
+  elem = g_list_find (device->slaves, slave);
+
+  if (!elem)
+    return;
+
+  device->slaves = g_list_delete_link (device->slaves, elem);
 }
 
 /**
@@ -798,18 +876,14 @@ _gdk_device_set_associated_device (GdkDevice *device,
 GdkDeviceType
 gdk_device_get_device_type (GdkDevice *device)
 {
-  GdkDevicePrivate *priv;
-
   g_return_val_if_fail (GDK_IS_DEVICE (device), GDK_DEVICE_TYPE_MASTER);
 
-  priv = device->priv;
-
-  return priv->type;
+  return device->type;
 }
 
 /**
  * gdk_device_get_n_axes:
- * @device: a #GdkDevice
+ * @device: a pointer #GdkDevice
  *
  * Returns the number of axes the device currently has.
  *
@@ -817,17 +891,18 @@ gdk_device_get_device_type (GdkDevice *device)
  *
  * Since: 3.0
  **/
-guint
+gint
 gdk_device_get_n_axes (GdkDevice *device)
 {
   g_return_val_if_fail (GDK_IS_DEVICE (device), 0);
+  g_return_val_if_fail (gdk_device_get_source (device) != GDK_SOURCE_KEYBOARD, 0);
 
-  return device->num_axes;
+  return device->axes->len;
 }
 
 /**
  * gdk_device_list_axes:
- * @device: a #GdkDevice
+ * @device: a pointer #GdkDevice
  *
  * Returns a #GList of #GdkAtom<!-- -->s, containing the labels for
  * the axes that @device currently has.
@@ -840,17 +915,17 @@ gdk_device_get_n_axes (GdkDevice *device)
 GList *
 gdk_device_list_axes (GdkDevice *device)
 {
-  GdkDevicePrivate *priv;
   GList *axes = NULL;
   gint i;
 
-  priv = device->priv;
+  g_return_val_if_fail (GDK_IS_DEVICE (device), NULL);
+  g_return_val_if_fail (gdk_device_get_source (device) != GDK_SOURCE_KEYBOARD, NULL);
 
-  for (i = 0; i < priv->axes->len; i++)
+  for (i = 0; i < device->axes->len; i++)
     {
       GdkAxisInfo axis_info;
 
-      axis_info = g_array_index (priv->axes, GdkAxisInfo, i);
+      axis_info = g_array_index (device->axes, GdkAxisInfo, i);
       axes = g_list_prepend (axes, GDK_ATOM_TO_POINTER (axis_info.label));
     }
 
@@ -859,7 +934,7 @@ gdk_device_list_axes (GdkDevice *device)
 
 /**
  * gdk_device_get_axis_value:
- * @device: a #GdkDevice.
+ * @device: a pointer #GdkDevice.
  * @axes: pointer to an array of axes
  * @axis_label: #GdkAtom with the axis label.
  * @value: location to store the found value.
@@ -878,21 +953,19 @@ gdk_device_get_axis_value (GdkDevice *device,
                            GdkAtom    axis_label,
                            gdouble   *value)
 {
-  GdkDevicePrivate *priv;
   gint i;
 
   g_return_val_if_fail (GDK_IS_DEVICE (device), FALSE);
+  g_return_val_if_fail (gdk_device_get_source (device) != GDK_SOURCE_KEYBOARD, FALSE);
 
   if (axes == NULL)
     return FALSE;
 
-  priv = device->priv;
-
-  for (i = 0; i < priv->axes->len; i++)
+  for (i = 0; i < device->axes->len; i++)
     {
       GdkAxisInfo axis_info;
 
-      axis_info = g_array_index (priv->axes, GdkAxisInfo, i);
+      axis_info = g_array_index (device->axes, GdkAxisInfo, i);
 
       if (axis_info.label != axis_label)
         continue;
@@ -924,23 +997,21 @@ gdk_device_get_axis (GdkDevice  *device,
                      GdkAxisUse  use,
                      gdouble    *value)
 {
-  GdkDevicePrivate *priv;
   gint i;
 
   g_return_val_if_fail (GDK_IS_DEVICE (device), FALSE);
+  g_return_val_if_fail (gdk_device_get_source (device) != GDK_SOURCE_KEYBOARD, FALSE);
 
   if (axes == NULL)
     return FALSE;
 
-  priv = device->priv;
+  g_return_val_if_fail (device->axes != NULL, FALSE);
 
-  g_return_val_if_fail (priv->axes != NULL, FALSE);
-
-  for (i = 0; i < priv->axes->len; i++)
+  for (i = 0; i < device->axes->len; i++)
     {
       GdkAxisInfo axis_info;
 
-      axis_info = g_array_index (priv->axes, GdkAxisInfo, i);
+      axis_info = g_array_index (device->axes, GdkAxisInfo, i);
 
       if (axis_info.use != use)
         continue;
@@ -1028,34 +1099,36 @@ gdk_device_grab (GdkDevice        *device,
   GdkGrabStatus res;
   GdkWindow *native;
 
-  g_return_val_if_fail (GDK_IS_DEVICE (device), 0);
-  g_return_val_if_fail (GDK_IS_WINDOW (window), 0);
+  g_return_val_if_fail (GDK_IS_DEVICE (device), GDK_GRAB_SUCCESS);
+  g_return_val_if_fail (GDK_IS_WINDOW (window), GDK_GRAB_SUCCESS);
 
   if (_gdk_native_windows)
     native = window;
   else
     native = gdk_window_get_toplevel (window);
 
-  while (((GdkWindowObject *) native)->window_type == GDK_WINDOW_OFFSCREEN)
+  while (native->window_type == GDK_WINDOW_OFFSCREEN)
     {
       native = gdk_offscreen_window_get_embedder (native);
 
       if (native == NULL ||
-	  (!_gdk_window_has_impl (native) &&
-	   !gdk_window_is_viewable (native)))
-	return GDK_GRAB_NOT_VIEWABLE;
+          (!_gdk_window_has_impl (native) &&
+           !gdk_window_is_viewable (native)))
+        return GDK_GRAB_NOT_VIEWABLE;
 
       native = gdk_window_get_toplevel (native);
     }
 
-  res = _gdk_windowing_device_grab (device,
-                                    window,
-                                    native,
-                                    owner_events,
-                                    get_native_grab_event_mask (event_mask),
-                                    NULL,
-                                    cursor,
-                                    time_);
+  if (native == NULL || GDK_WINDOW_DESTROYED (native))
+    return GDK_GRAB_NOT_VIEWABLE;
+
+  res = GDK_DEVICE_GET_CLASS (device)->grab (device,
+                                             native,
+                                             owner_events,
+                                             get_native_grab_event_mask (event_mask),
+                                             NULL,
+                                             cursor,
+                                             time_);
 
   if (res == GDK_GRAB_SUCCESS)
     {
@@ -1063,7 +1136,7 @@ gdk_device_grab (GdkDevice        *device,
       gulong serial;
 
       display = gdk_window_get_display (window);
-      serial = _gdk_windowing_window_get_next_serial (display);
+      serial = _gdk_display_get_next_serial (display);
 
       _gdk_display_add_device_grab (display,
                                     device,
@@ -1080,23 +1153,68 @@ gdk_device_grab (GdkDevice        *device,
   return res;
 }
 
+/**
+ * gdk_device_ungrab:
+ * @device: a #GdkDevice
+ * @time_: a timestap (e.g. %GDK_CURRENT_TIME).
+ *
+ * Release any grab on @device.
+ *
+ * Since: 3.0
+ */
+void
+gdk_device_ungrab (GdkDevice  *device,
+                   guint32     time_)
+{
+  g_return_if_fail (GDK_IS_DEVICE (device));
+
+  GDK_DEVICE_GET_CLASS (device)->ungrab (device, time_);
+}
+
+/**
+ * gdk_device_warp:
+ * @device: the device to warp.
+ * @screen: the screen to warp @device to.
+ * @x: the X coordinate of the destination.
+ * @y: the Y coordinate of the destination.
+ *
+ * Warps @device in @display to the point @x,@y on
+ * the screen @screen, unless the device is confined
+ * to a window by a grab, in which case it will be moved
+ * as far as allowed by the grab. Warping the pointer
+ * creates events as if the user had moved the mouse
+ * instantaneously to the destination.
+ *
+ * Note that the pointer should normally be under the
+ * control of the user. This function was added to cover
+ * some rare use cases like keyboard navigation support
+ * for the color picker in the #GtkColorSelectionDialog.
+ *
+ * Since: 3.0
+ **/
+void
+gdk_device_warp (GdkDevice  *device,
+                 GdkScreen  *screen,
+                 gint        x,
+                 gint        y)
+{
+  g_return_if_fail (GDK_IS_DEVICE (device));
+  g_return_if_fail (GDK_IS_SCREEN (screen));
+  g_return_if_fail (gdk_device_get_display (device) == gdk_screen_get_display (screen));
+
+  GDK_DEVICE_GET_CLASS (device)->warp (device, screen, x, y);
+}
+
 /* Private API */
 void
 _gdk_device_reset_axes (GdkDevice *device)
 {
-  GdkDevicePrivate *priv;
   gint i;
 
-  priv = device->priv;
-
-  for (i = priv->axes->len - 1; i >= 0; i--)
-    g_array_remove_index (priv->axes, i);
+  for (i = device->axes->len - 1; i >= 0; i--)
+    g_array_remove_index (device->axes, i);
 
   g_object_notify (G_OBJECT (device), "n-axes");
-
-  /* This is done for backwards compatibility */
-  g_free (device->axes);
-  device->axes = NULL;
 }
 
 guint
@@ -1107,12 +1225,9 @@ _gdk_device_add_axis (GdkDevice   *device,
                       gdouble      max_value,
                       gdouble      resolution)
 {
-  GdkDevicePrivate *priv;
   GdkAxisInfo axis_info;
   guint pos;
 
-  priv = device->priv;
-
   axis_info.use = use;
   axis_info.label = label_atom;
   axis_info.min_value = min_value;
@@ -1137,17 +1252,8 @@ _gdk_device_add_axis (GdkDevice   *device,
       break;
     }
 
-  priv->axes = g_array_append_val (priv->axes, axis_info);
-  device->num_axes = priv->axes->len;
-  pos = device->num_axes - 1;
-
-  /* This is done for backwards compatibility, since the public
-   * struct doesn't actually store the device data.
-   */
-  device->axes = g_realloc (device->axes, sizeof (GdkDeviceAxis) * priv->axes->len);
-  device->axes[pos].use = axis_info.use;
-  device->axes[pos].min = axis_info.min_axis;
-  device->axes[pos].max = axis_info.max_axis;
+  device->axes = g_array_append_val (device->axes, axis_info);
+  pos = device->axes->len - 1;
 
   g_object_notify (G_OBJECT (device), "n-axes");
 
@@ -1187,12 +1293,9 @@ GdkAxisUse
 _gdk_device_get_axis_use (GdkDevice *device,
                           guint      index_)
 {
-  GdkDevicePrivate *priv;
   GdkAxisInfo info;
 
-  priv = device->priv;
-
-  info = g_array_index (priv->axes, GdkAxisInfo, index_);
+  info = g_array_index (device->axes, GdkAxisInfo, index_);
   return info.use;
 }
 
@@ -1203,7 +1306,6 @@ _gdk_device_translate_window_coord (GdkDevice *device,
                                     gdouble    value,
                                     gdouble   *axis_value)
 {
-  GdkDevicePrivate *priv;
   GdkAxisInfo axis_info;
   GdkAxisInfo *axis_info_x, *axis_info_y;
   gdouble device_width, device_height;
@@ -1213,14 +1315,11 @@ _gdk_device_translate_window_coord (GdkDevice *device,
   gdouble x_resolution, y_resolution;
   gdouble device_aspect;
   gint window_width, window_height;
-  GdkWindowObject *window_private;
 
-  priv = device->priv;
-
-  if (index_ >= priv->axes->len)
+  if (index_ >= device->axes->len)
     return FALSE;
 
-  axis_info = g_array_index (priv->axes, GdkAxisInfo, index_);
+  axis_info = g_array_index (device->axes, GdkAxisInfo, index_);
 
   if (axis_info.use != GDK_AXIS_X &&
       axis_info.use != GDK_AXIS_Y)
@@ -1229,11 +1328,11 @@ _gdk_device_translate_window_coord (GdkDevice *device,
   if (axis_info.use == GDK_AXIS_X)
     {
       axis_info_x = &axis_info;
-      axis_info_y = find_axis_info (priv->axes, GDK_AXIS_Y);
+      axis_info_y = find_axis_info (device->axes, GDK_AXIS_Y);
     }
   else
     {
-      axis_info_x = find_axis_info (priv->axes, GDK_AXIS_X);
+      axis_info_x = find_axis_info (device->axes, GDK_AXIS_X);
       axis_info_y = &axis_info;
     }
 
@@ -1256,7 +1355,6 @@ _gdk_device_translate_window_coord (GdkDevice *device,
       y_min = 0;
     }
 
-  window_private = (GdkWindowObject *) window;
   window_width = gdk_window_get_width (window);
   window_height = gdk_window_get_height (window);
 
@@ -1321,27 +1419,22 @@ _gdk_device_translate_screen_coord (GdkDevice *device,
                                     gdouble    value,
                                     gdouble   *axis_value)
 {
-  GdkDevicePrivate *priv;
   GdkAxisInfo axis_info;
   gdouble axis_width, scale, offset;
-  GdkWindowObject *window_private;
 
   if (device->mode != GDK_MODE_SCREEN)
     return FALSE;
 
-  priv = device->priv;
-
-  if (index_ >= priv->axes->len)
+  if (index_ >= device->axes->len)
     return FALSE;
 
-  axis_info = g_array_index (priv->axes, GdkAxisInfo, index_);
+  axis_info = g_array_index (device->axes, GdkAxisInfo, index_);
 
   if (axis_info.use != GDK_AXIS_X &&
       axis_info.use != GDK_AXIS_Y)
     return FALSE;
 
   axis_width = axis_info.max_value - axis_info.min_value;
-  window_private = (GdkWindowObject *) window;
 
   if (axis_info.use == GDK_AXIS_X)
     {
@@ -1350,7 +1443,7 @@ _gdk_device_translate_screen_coord (GdkDevice *device,
       else
         scale = 1;
 
-      offset = - window_root_x - window_private->abs_x;
+      offset = - window_root_x - window->abs_x;
     }
   else
     {
@@ -1359,7 +1452,7 @@ _gdk_device_translate_screen_coord (GdkDevice *device,
       else
         scale = 1;
 
-      offset = - window_root_y - window_private->abs_y;
+      offset = - window_root_y - window->abs_y;
     }
 
   if (axis_value)
@@ -1374,16 +1467,13 @@ _gdk_device_translate_axis (GdkDevice *device,
                             gdouble    value,
                             gdouble   *axis_value)
 {
-  GdkDevicePrivate *priv;
   GdkAxisInfo axis_info;
   gdouble axis_width, out;
 
-  priv = device->priv;
-
-  if (index_ >= priv->axes->len)
+  if (index_ >= device->axes->len)
     return FALSE;
 
-  axis_info = g_array_index (priv->axes, GdkAxisInfo, index_);
+  axis_info = g_array_index (device->axes, GdkAxisInfo, index_);
 
   if (axis_info.use == GDK_AXIS_X ||
       axis_info.use == GDK_AXIS_Y)
@@ -1398,3 +1488,39 @@ _gdk_device_translate_axis (GdkDevice *device,
 
   return TRUE;
 }
+
+gboolean
+_gdk_device_query_state (GdkDevice        *device,
+                         GdkWindow        *window,
+                         GdkWindow       **root_window,
+                         GdkWindow       **child_window,
+                         gint             *root_x,
+                         gint             *root_y,
+                         gint             *win_x,
+                         gint             *win_y,
+                         GdkModifierType  *mask)
+{
+  return GDK_DEVICE_GET_CLASS (device)->query_state (device,
+                                                     window,
+                                                     root_window,
+                                                     child_window,
+                                                     root_x,
+                                                     root_y,
+                                                     win_x,
+                                                     win_y,
+                                                     mask);
+}
+
+GdkWindow *
+_gdk_device_window_at_position (GdkDevice        *device,
+                                gint             *win_x,
+                                gint             *win_y,
+                                GdkModifierType  *mask,
+                                gboolean          get_toplevel)
+{
+  return GDK_DEVICE_GET_CLASS (device)->window_at_position (device,
+                                                            win_x,
+                                                            win_y,
+                                                            mask,
+                                                            get_toplevel);
+}
