@@ -1,7 +1,7 @@
 /*
  * gdkdisplay-x11.h
- * 
- * Copyright 2001 Sun Microsystems Inc. 
+ *
+ * Copyright 2001 Sun Microsystems Inc.
  *
  * Erwann Chenede <erwann.chenede@sun.com>
  *
@@ -21,31 +21,22 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifndef __GDK_DISPLAY_X11__
-#define __GDK_DISPLAY_X11__
+#ifndef __GDK_X11_DISPLAY__
+#define __GDK_X11_DISPLAY__
+
+#include "gdkdisplayprivate.h"
+#include "gdkkeys.h"
+#include "gdkwindow.h"
+#include "gdkinternals.h"
+#include "gdkmain.h"
 
 #include <X11/X.h>
 #include <X11/Xlib.h>
-#include <glib.h>
-#include <gdk/gdkdisplay.h>
-#include <gdk/gdkkeys.h>
-#include <gdk/gdkwindow.h>
-#include <gdk/gdkinternals.h>
-#include <gdk/gdk.h>		/* For gdk_get_program_class() */
 
 G_BEGIN_DECLS
 
-typedef struct _GdkDisplayX11 GdkDisplayX11;
-typedef struct _GdkDisplayX11Class GdkDisplayX11Class;
 
-#define GDK_TYPE_DISPLAY_X11              (_gdk_display_x11_get_type())
-#define GDK_DISPLAY_X11(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), GDK_TYPE_DISPLAY_X11, GdkDisplayX11))
-#define GDK_DISPLAY_X11_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), GDK_TYPE_DISPLAY_X11, GdkDisplayX11Class))
-#define GDK_IS_DISPLAY_X11(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), GDK_TYPE_DISPLAY_X11))
-#define GDK_IS_DISPLAY_X11_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), GDK_TYPE_DISPLAY_X11))
-#define GDK_DISPLAY_X11_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), GDK_TYPE_DISPLAY_X11, GdkDisplayX11Class))
-
-struct _GdkDisplayX11
+struct _GdkX11Display
 {
   GdkDisplay parent_instance;
   Display *xdisplay;
@@ -57,17 +48,17 @@ struct _GdkDisplayX11
   gint grab_count;
 
   /* Keyboard related information */
-
   gint xkb_event_type;
   gboolean use_xkb;
-  
+
   /* Whether we were able to turn on detectable-autorepeat using
    * XkbSetDetectableAutorepeat. If FALSE, we'll fall back
-   * to checking the next event with XPending(). */
+   * to checking the next event with XPending().
+   */
   gboolean have_xkb_autorepeat;
 
   GdkKeymap *keymap;
-  guint	    keymap_serial;
+  guint      keymap_serial;
 
   gboolean have_xfixes;
   gint xfixes_event_base;
@@ -79,23 +70,22 @@ struct _GdkDisplayX11
   gboolean have_randr13;
   gint xrandr_event_base;
 
-  /* If the SECURITY extension is in place, whether this client holds 
-   * a trusted authorization and so is allowed to make various requests 
-   * (grabs, properties etc.) Otherwise always TRUE. */
+  /* If the SECURITY extension is in place, whether this client holds
+   * a trusted authorization and so is allowed to make various requests
+   * (grabs, properties etc.) Otherwise always TRUE.
+   */
   gboolean trusted_client;
 
   /* drag and drop information */
   GdkDragContext *current_dest_drag;
 
   /* data needed for MOTIF DnD */
-
   Window motif_drag_window;
   GdkWindow *motif_drag_gdk_window;
   GList **motif_target_lists;
   gint motif_n_target_lists;
 
   /* Mapping to/from virtual atoms */
-
   GHashTable *atom_from_virtual;
   GHashTable *atom_to_virtual;
 
@@ -103,13 +93,13 @@ struct _GdkDisplayX11
   Window leader_window;
   GdkWindow *leader_gdk_window;
   gboolean leader_window_title_set;
-  
+
   /* list of filters for client messages */
   GList *client_filters;
 
   /* List of functions to go from extension event => X window */
   GSList *event_types;
-  
+
   /* X ID hashtable */
   GHashTable *xid_ht;
 
@@ -141,17 +131,20 @@ struct _GdkDisplayX11
 
   /* The offscreen window that has the pointer in it (if any) */
   GdkWindow *active_offscreen_window;
+
+  GSList *error_traps;
 };
 
-struct _GdkDisplayX11Class
+struct _GdkX11DisplayClass
 {
   GdkDisplayClass parent_class;
 };
 
-GType      _gdk_display_x11_get_type            (void);
-GdkScreen *_gdk_x11_display_screen_for_xrootwin (GdkDisplay *display,
-						 Window      xrootwin);
+GdkScreen *_gdk_x11_display_screen_for_xrootwin (GdkDisplay  *display,
+                                                 Window       xrootwin);
+void       _gdk_x11_display_error_event         (GdkDisplay  *display,
+                                                 XErrorEvent *error);
 
 G_END_DECLS
 
-#endif				/* __GDK_DISPLAY_X11__ */
+#endif  /* __GDK_X11_DISPLAY__ */
