@@ -42,7 +42,6 @@
 #include "gtkbuildable.h"
 #include "gtktypeutils.h"
 #include "gtkintl.h"
-#include "gtkalias.h"
 
 
 typedef GtkBuildableIface GtkBuildableInterface;
@@ -205,12 +204,12 @@ gtk_buildable_parser_finished (GtkBuildable *buildable,
  * @builder: #GtkBuilder used to construct this object
  * @name: name of child to construct
  *
- * Constructs a child of @buildable with the name @name. 
+ * Constructs a child of @buildable with the name @name.
  *
  * #GtkBuilder calls this function if a "constructor" has been
  * specified in the UI definition.
  *
- * Returns: the constructed child
+ * Returns: (transfer full): the constructed child
  *
  * Since: 2.12
  **/
@@ -338,7 +337,7 @@ gtk_buildable_custom_finished (GtkBuildable  *buildable,
  *
  * Get the internal child called @childname of the @buildable object.
  *
- * Returns: the internal child of the buildable object 
+ * Returns: (transfer none): the internal child of the buildable object
  *
  * Since: 2.12
  **/
@@ -359,6 +358,3 @@ gtk_buildable_get_internal_child (GtkBuildable *buildable,
 
   return (* iface->get_internal_child) (buildable, builder, childname);
 }
-
-#define __GTK_BUILDABLE_C__
-#include "gtkaliasdef.c"
