@@ -132,11 +132,9 @@ struct _GtkImagePrivate
 {
   GtkIconSize           icon_size;      /* Only used with GTK_IMAGE_STOCK, GTK_IMAGE_ICON_SET, GTK_IMAGE_ICON_NAME */
   GtkImageType          storage_type;
-  GdkBitmap            *mask;           /* Only used with GTK_IMAGE_PIXMAP, GTK_IMAGE_IMAGE */
 
   union
   {
-    GtkImagePixmapData     pixmap;
     GtkImagePixbufData     pixbuf;
     GtkImageStockData      stock;
     GtkImageIconSetData    icon_set;
@@ -157,41 +155,44 @@ struct _GtkImagePrivate
 
 
 #define DEFAULT_ICON_SIZE GTK_ICON_SIZE_BUTTON
-static gint gtk_image_expose       (GtkWidget      *widget,
-                                    GdkEventExpose *event);
-static void gtk_image_unmap        (GtkWidget      *widget);
-static void gtk_image_unrealize    (GtkWidget      *widget);
-static void gtk_image_size_request (GtkWidget      *widget,
-                                    GtkRequisition *requisition);
-static void gtk_image_style_set    (GtkWidget      *widget,
-				    GtkStyle       *prev_style);
-static void gtk_image_screen_changed (GtkWidget    *widget,
-				      GdkScreen    *prev_screen);
-static void gtk_image_destroy      (GtkObject      *object);
-static void gtk_image_reset        (GtkImage       *image);
-static void gtk_image_calc_size    (GtkImage       *image);
+static gint gtk_image_draw                 (GtkWidget    *widget,
+                                            cairo_t      *cr);
+static void gtk_image_unmap                (GtkWidget    *widget);
+static void gtk_image_unrealize            (GtkWidget    *widget);
+static void gtk_image_get_preferred_width  (GtkWidget    *widget,
+                                            gint         *minimum,
+                                            gint         *natural);
+static void gtk_image_get_preferred_height (GtkWidget    *widget,
+                                            gint         *minimum,
+                                            gint         *natural);
 
-static void gtk_image_update_size  (GtkImage       *image,
-                                    gint            image_width,
-                                    gint            image_height);
+static void gtk_image_style_set            (GtkWidget    *widget,
+                                            GtkStyle     *prev_style);
+static void gtk_image_screen_changed       (GtkWidget    *widget,
+                                            GdkScreen    *prev_screen);
+static void gtk_image_destroy              (GtkWidget    *widget);
+static void gtk_image_reset                (GtkImage     *image);
+static void gtk_image_calc_size            (GtkImage     *image);
 
-static void gtk_image_set_property      (GObject          *object,
-					 guint             prop_id,
-					 const GValue     *value,
-					 GParamSpec       *pspec);
-static void gtk_image_get_property      (GObject          *object,
-					 guint             prop_id,
-					 GValue           *value,
-					 GParamSpec       *pspec);
+static void gtk_image_update_size          (GtkImage     *image,
+                                            gint          image_width,
+                                            gint          image_height);
 
-static void icon_theme_changed          (GtkImage         *image);
+static void gtk_image_set_property         (GObject      *object,
+                                            guint         prop_id,
+                                            const GValue *value,
+                                            GParamSpec   *pspec);
+static void gtk_image_get_property         (GObject      *object,
+                                            guint         prop_id,
+                                            GValue       *value,
+                                            GParamSpec   *pspec);
+
+static void icon_theme_changed             (GtkImage     *image);
 
 enum
 {
   PROP_0,
   PROP_PIXBUF,
-  PROP_PIXMAP,
-  PROP_MASK,
   PROP_FILE,
   PROP_STOCK,
   PROP_ICON_SET,
@@ -209,22 +210,18 @@ static void
 gtk_image_class_init (GtkImageClass *class)
 {
   GObjectClass *gobject_class;
-  GtkObjectClass *object_class;
   GtkWidgetClass *widget_class;
 
   gobject_class = G_OBJECT_CLASS (class);
   
   gobject_class->set_property = gtk_image_set_property;
   gobject_class->get_property = gtk_image_get_property;
-  
-  object_class = GTK_OBJECT_CLASS (class);
-  
-  object_class->destroy = gtk_image_destroy;
 
   widget_class = GTK_WIDGET_CLASS (class);
-  
-  widget_class->expose_event = gtk_image_expose;
-  widget_class->size_request = gtk_image_size_request;
+  widget_class->draw = gtk_image_draw;
+  widget_class->destroy = gtk_image_destroy;
+  widget_class->get_preferred_width = gtk_image_get_preferred_width;
+  widget_class->get_preferred_height = gtk_image_get_preferred_height;
   widget_class->unmap = gtk_image_unmap;
   widget_class->unrealize = gtk_image_unrealize;
   widget_class->style_set = gtk_image_style_set;
@@ -238,22 +235,6 @@ gtk_image_class_init (GtkImageClass *class)
                                                         GDK_TYPE_PIXBUF,
                                                         GTK_PARAM_READWRITE));
 
-  g_object_class_install_property (gobject_class,
-                                   PROP_PIXMAP,
-                                   g_param_spec_object ("pixmap",
-                                                        P_("Pixmap"),
-                                                        P_("A GdkPixmap to display"),
-                                                        GDK_TYPE_PIXMAP,
-                                                        GTK_PARAM_READWRITE));
-
-  g_object_class_install_property (gobject_class,
-                                   PROP_MASK,
-                                   g_param_spec_object ("mask",
-                                                        P_("Mask"),
-                                                        P_("Mask bitmap to use with GdkPixmap"),
-                                                        GDK_TYPE_PIXMAP,
-                                                        GTK_PARAM_READWRITE));
-  
   g_object_class_install_property (gobject_class,
                                    PROP_FILE,
                                    g_param_spec_string ("file",
@@ -355,7 +336,7 @@ gtk_image_class_init (GtkImageClass *class)
                                                       GTK_IMAGE_EMPTY,
                                                       GTK_PARAM_READABLE));
 
-  g_type_class_add_private (object_class, sizeof (GtkImagePrivate));
+  g_type_class_add_private (class, sizeof (GtkImagePrivate));
 }
 
 static void
@@ -372,7 +353,6 @@ gtk_image_init (GtkImage *image)
 
   priv->storage_type = GTK_IMAGE_EMPTY;
   priv->icon_size = DEFAULT_ICON_SIZE;
-  priv->mask = NULL;
 
   priv->pixel_size = -1;
 
@@ -380,13 +360,13 @@ gtk_image_init (GtkImage *image)
 }
 
 static void
-gtk_image_destroy (GtkObject *object)
+gtk_image_destroy (GtkWidget *widget)
 {
-  GtkImage *image = GTK_IMAGE (object);
+  GtkImage *image = GTK_IMAGE (widget);
 
   gtk_image_reset (image);
-  
-  GTK_OBJECT_CLASS (gtk_image_parent_class)->destroy (object);
+
+  GTK_WIDGET_CLASS (gtk_image_parent_class)->destroy (widget);
 }
 
 static void 
@@ -404,30 +384,6 @@ gtk_image_set_property (GObject      *object,
       gtk_image_set_from_pixbuf (image,
                                  g_value_get_object (value));
       break;
-    case PROP_PIXMAP:
-      gtk_image_set_from_pixmap (image,
-                                 g_value_get_object (value),
-                                 priv->mask);
-      break;
-    case PROP_MASK:
-      if (priv->storage_type == GTK_IMAGE_PIXMAP)
-        gtk_image_set_from_pixmap (image,
-                                   priv->data.pixmap.pixmap,
-                                   g_value_get_object (value));
-      else
-        {
-          GdkBitmap *mask;
-
-          mask = g_value_get_object (value);
-
-          if (mask)
-            g_object_ref (mask);
-
-          gtk_image_clear (image);
-
-          priv->mask = mask;
-        }
-      break;
     case PROP_FILE:
       gtk_image_set_from_file (image, g_value_get_string (value));
       break;
@@ -506,16 +462,6 @@ gtk_image_get_property (GObject     *object,
         g_value_set_object (value,
                             gtk_image_get_pixbuf (image));
       break;
-    case PROP_PIXMAP:
-      if (priv->storage_type != GTK_IMAGE_PIXMAP)
-        g_value_set_object (value, NULL);
-      else
-        g_value_set_object (value,
-                            priv->data.pixmap.pixmap);
-      break;
-    case PROP_MASK:
-      g_value_set_object (value, priv->mask);
-      break;
     case PROP_FILE:
       g_value_set_string (value, priv->filename);
       break;
@@ -571,32 +517,6 @@ gtk_image_get_property (GObject     *object,
 }
 
 
-/**
- * gtk_image_new_from_pixmap:
- * @pixmap: (allow-none): a #GdkPixmap, or %NULL
- * @mask: (allow-none): a #GdkBitmap, or %NULL
- *
- * Creates a #GtkImage widget displaying @pixmap with a @mask.
- * A #GdkPixmap is a server-side image buffer in the pixel format of the
- * current display. The #GtkImage does not assume a reference to the
- * pixmap or mask; you still need to unref them if you own references.
- * #GtkImage will add its own reference rather than adopting yours.
- * 
- * Return value: a new #GtkImage
- **/
-GtkWidget*
-gtk_image_new_from_pixmap (GdkPixmap *pixmap,
-                           GdkBitmap *mask)
-{
-  GtkImage *image;
-
-  image = g_object_new (GTK_TYPE_IMAGE, NULL);
-
-  gtk_image_set_from_pixmap (image, pixmap, mask);
-
-  return GTK_WIDGET (image);
-}
-
 /**
  * gtk_image_new_from_file:
  * @filename: a filename
@@ -802,61 +722,6 @@ gtk_image_new_from_gicon (GIcon *icon,
   return GTK_WIDGET (image);
 }
 
-/**
- * gtk_image_set_from_pixmap:
- * @image: a #GtkImage
- * @pixmap: (allow-none): a #GdkPixmap or %NULL
- * @mask: (allow-none): a #GdkBitmap or %NULL
- *
- * See gtk_image_new_from_pixmap() for details.
- **/
-void
-gtk_image_set_from_pixmap (GtkImage  *image,
-                           GdkPixmap *pixmap,
-                           GdkBitmap *mask)
-{
-  GtkImagePrivate *priv;
-
-  g_return_if_fail (GTK_IS_IMAGE (image));
-  g_return_if_fail (pixmap == NULL ||
-                    GDK_IS_PIXMAP (pixmap));
-  g_return_if_fail (mask == NULL ||
-                    GDK_IS_PIXMAP (mask));
-
-  priv = image->priv;
-
-  g_object_freeze_notify (G_OBJECT (image));
-  
-  if (pixmap)
-    g_object_ref (pixmap);
-
-  if (mask)
-    g_object_ref (mask);
-
-  gtk_image_clear (image);
-
-  priv->mask = mask;
-
-  if (pixmap)
-    {
-      int width;
-      int height;
-
-      priv->storage_type = GTK_IMAGE_PIXMAP;
-
-      priv->data.pixmap.pixmap = pixmap;
-
-      gdk_drawable_get_size (GDK_DRAWABLE (pixmap), &width, &height);
-
-      gtk_image_update_size (image, width, height);
-    }
-
-  g_object_notify (G_OBJECT (image), "pixmap");
-  g_object_notify (G_OBJECT (image), "mask");
-  
-  g_object_thaw_notify (G_OBJECT (image));
-}
-
 /**
  * gtk_image_set_from_file:
  * @image: a #GtkImage
@@ -1206,41 +1071,6 @@ gtk_image_get_storage_type (GtkImage *image)
   return image->priv->storage_type;
 }
 
-/**
- * gtk_image_get_pixmap:
- * @image: a #GtkImage
- * @pixmap: (out) (transfer none) (allow-none): location to store the
- *     pixmap, or %NULL
- * @mask: (out) (transfer none) (allow-none): location to store the
- *     mask, or %NULL
- *
- * Gets the pixmap and mask being displayed by the #GtkImage.
- * The storage type of the image must be %GTK_IMAGE_EMPTY or
- * %GTK_IMAGE_PIXMAP (see gtk_image_get_storage_type()).
- * The caller of this function does not own a reference to the
- * returned pixmap and mask.
- **/
-void
-gtk_image_get_pixmap (GtkImage   *image,
-                      GdkPixmap **pixmap,
-                      GdkBitmap **mask)
-{
-  GtkImagePrivate *priv;
-
-  g_return_if_fail (GTK_IS_IMAGE (image));
-
-  priv = image->priv;
-
-  g_return_if_fail (priv->storage_type == GTK_IMAGE_PIXMAP ||
-                    priv->storage_type == GTK_IMAGE_EMPTY);
-
-  if (pixmap)
-    *pixmap = priv->data.pixmap.pixmap;
-
-  if (mask)
-    *mask = priv->mask;
-}
-
 /**
  * gtk_image_get_pixbuf:
  * @image: a #GtkImage
@@ -1639,15 +1469,15 @@ ensure_pixbuf_for_icon_name (GtkImage     *image,
                                          MIN (width, height), flags);
       if (info)
         {
-          GtkStyle *style;
+          GtkStyleContext *context;
           gboolean was_symbolic;
 
-          style = gtk_widget_get_style (GTK_WIDGET (image));
+          context = gtk_widget_get_style_context (GTK_WIDGET (image));
           priv->data.name.pixbuf =
-            gtk_icon_info_load_symbolic_for_style (info,
-                                                   style, state,
-                                                   &was_symbolic,
-                                                   NULL);
+            gtk_icon_info_load_symbolic_for_context (info,
+                                                     context,
+                                                     &was_symbolic,
+                                                     NULL);
           priv->was_symbolic = was_symbolic;
           gtk_icon_info_free (info);
         }
@@ -1655,10 +1485,9 @@ ensure_pixbuf_for_icon_name (GtkImage     *image,
       if (priv->data.name.pixbuf == NULL)
 	{
 	  priv->data.name.pixbuf =
-	    gtk_widget_render_icon (GTK_WIDGET (image),
-				    GTK_STOCK_MISSING_IMAGE,
-				    priv->icon_size,
-				    NULL);
+	    gtk_widget_render_icon_pixbuf (GTK_WIDGET (image),
+                                           GTK_STOCK_MISSING_IMAGE,
+                                           priv->icon_size);
 	  priv->was_symbolic = FALSE;
 	}
     }
@@ -1714,15 +1543,15 @@ ensure_pixbuf_for_gicon (GtkImage     *image,
 					     MIN (width, height), flags);
       if (info)
         {
-          GtkStyle *style;
+          GtkStyleContext *context;
           gboolean was_symbolic;
 
-          style = gtk_widget_get_style (GTK_WIDGET (image));
+          context = gtk_widget_get_style_context (GTK_WIDGET (image));
           priv->data.gicon.pixbuf =
-            gtk_icon_info_load_symbolic_for_style (info,
-                                                   style, state,
-                                                   &was_symbolic,
-                                                   NULL);
+            gtk_icon_info_load_symbolic_for_context (info,
+                                                     context,
+                                                     &was_symbolic,
+                                                     NULL);
           priv->was_symbolic = was_symbolic;
           gtk_icon_info_free (info);
         }
@@ -1730,83 +1559,37 @@ ensure_pixbuf_for_gicon (GtkImage     *image,
       if (priv->data.gicon.pixbuf == NULL)
 	{
 	  priv->data.gicon.pixbuf =
-	    gtk_widget_render_icon (GTK_WIDGET (image),
-				    GTK_STOCK_MISSING_IMAGE,
-				    priv->icon_size,
-				    NULL);
+	    gtk_widget_render_icon_pixbuf (GTK_WIDGET (image),
+                                           GTK_STOCK_MISSING_IMAGE,
+                                           priv->icon_size);
 	  priv->was_symbolic = FALSE;
 	}
     }
 }
 
-
-/*
- * Like gdk_rectangle_intersect (dest, src, dest), but make 
- * sure that the origin of dest is moved by an "even" offset. 
- * If necessary grow the intersection by one row or column 
- * to achieve this.
- *
- * This is necessary since we can't pass alignment information
- * for the pixelation pattern down to gdk_pixbuf_saturate_and_pixelate(), 
- * thus we have to makesure that the subimages are properly aligned.
- */
-static gboolean
-rectangle_intersect_even (GdkRectangle *src, 
-			  GdkRectangle *dest)
-{
-  gboolean isect;
-  gint x, y;
-
-  x = dest->x;
-  y = dest->y;
-  isect = gdk_rectangle_intersect (dest, src, dest);
-
-  if ((dest->x - x + dest->y - y) % 2 != 0)
-    {
-      if (dest->x > x)
-	{
-	  dest->x--;
-	  dest->width++;
-	}
-      else
-	{
-	  dest->y--;
-	  dest->height++;
-	}
-    }
-  
-  return isect;
-}
-
 static gint
-gtk_image_expose (GtkWidget      *widget,
-		  GdkEventExpose *event)
+gtk_image_draw (GtkWidget *widget,
+                cairo_t   *cr)
 {
   GtkImage *image;
   GtkImagePrivate *priv;
 
   g_return_val_if_fail (GTK_IS_IMAGE (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
 
   image = GTK_IMAGE (widget);
   priv = image->priv;
   
-  if (gtk_widget_get_mapped (widget) &&
-      priv->storage_type != GTK_IMAGE_EMPTY)
+  if (priv->storage_type != GTK_IMAGE_EMPTY)
     {
-      GtkAllocation allocation;
       GtkMisc *misc;
-      GdkRectangle area, image_bound;
-      gint x, y, mask_x, mask_y;
+      gint x, y;
       gint xpad, ypad;
       gfloat xalign, yalign;
-      GdkBitmap *mask;
       GdkPixbuf *pixbuf;
       GtkStateType state;
       gboolean needs_state_transform;
 
       misc = GTK_MISC (widget);
-      area = event->area;
 
       /* For stock items and icon sets, we lazily calculate
        * the size; we might get here between a queue_resize()
@@ -1816,86 +1599,29 @@ gtk_image_expose (GtkWidget      *widget,
       if (priv->need_calc_size)
 	gtk_image_calc_size (image);
 
-      gtk_widget_get_allocation (widget, &allocation);
-
-      if (!gdk_rectangle_intersect (&area, &allocation, &area))
-	return FALSE;
-
       gtk_misc_get_alignment (misc, &xalign, &yalign);
       gtk_misc_get_padding (misc, &xpad, &ypad);
 
       if (gtk_widget_get_direction (widget) != GTK_TEXT_DIR_LTR)
 	xalign = 1.0 - xalign;
 
-      x = floor (allocation.x + xpad + ((allocation.width - priv->required_width) * xalign));
-      y = floor (allocation.y + ypad + ((allocation.height - priv->required_height) * yalign));
-      mask_x = x;
-      mask_y = y;
+      x = floor (xpad + ((gtk_widget_get_allocated_width (widget) - priv->required_width) * xalign));
+      y = floor (ypad + ((gtk_widget_get_allocated_height (widget) - priv->required_height) * yalign));
       
-      image_bound.x = x;
-      image_bound.y = y;      
-      image_bound.width = 0;
-      image_bound.height = 0;      
-
-      mask = NULL;
-      pixbuf = NULL;
       needs_state_transform = gtk_widget_get_state (widget) != GTK_STATE_NORMAL;
       
       switch (priv->storage_type)
         {
-        case GTK_IMAGE_PIXMAP:
-          mask = priv->mask;
-          gdk_drawable_get_size (priv->data.pixmap.pixmap,
-                                 &image_bound.width,
-                                 &image_bound.height);
-	  if (rectangle_intersect_even (&area, &image_bound) &&
-	      needs_state_transform)
-            {
-              pixbuf = gdk_pixbuf_get_from_drawable (NULL,
-                                                     priv->data.pixmap.pixmap,
-                                                     gtk_widget_get_colormap (widget),
-                                                     image_bound.x - x, image_bound.y - y,
-						     0, 0,
-                                                     image_bound.width,
-                                                     image_bound.height);
-
-	      x = image_bound.x;
-	      y = image_bound.y;
-            }
-	  
-          break;
 
         case GTK_IMAGE_PIXBUF:
-          image_bound.width = gdk_pixbuf_get_width (priv->data.pixbuf.pixbuf);
-          image_bound.height = gdk_pixbuf_get_height (priv->data.pixbuf.pixbuf);
-
-	  if (rectangle_intersect_even (&area, &image_bound) &&
-	      needs_state_transform)
-	    {
-	      pixbuf = gdk_pixbuf_new_subpixbuf (priv->data.pixbuf.pixbuf,
-						 image_bound.x - x, image_bound.y - y,
-						 image_bound.width, image_bound.height);
-
-	      x = image_bound.x;
-	      y = image_bound.y;
-	    }
-	  else
-	    {
-	      pixbuf = priv->data.pixbuf.pixbuf;
-	      g_object_ref (pixbuf);
-	    }
+          pixbuf = priv->data.pixbuf.pixbuf;
+          g_object_ref (pixbuf);
           break;
 
         case GTK_IMAGE_STOCK:
-          pixbuf = gtk_widget_render_icon (widget,
-                                           priv->data.stock.stock_id,
-                                           priv->icon_size,
-                                           NULL);
-          if (pixbuf)
-            {              
-              image_bound.width = gdk_pixbuf_get_width (pixbuf);
-              image_bound.height = gdk_pixbuf_get_height (pixbuf);
-            }
+          pixbuf = gtk_widget_render_icon_pixbuf (widget,
+                                                  priv->data.stock.stock_id,
+                                                  priv->icon_size);
 
           /* already done */
           needs_state_transform = FALSE;
@@ -1911,12 +1637,6 @@ gtk_image_expose (GtkWidget      *widget,
                                       widget,
                                       NULL);
 
-          if (pixbuf)
-            {
-              image_bound.width = gdk_pixbuf_get_width (pixbuf);
-              image_bound.height = gdk_pixbuf_get_height (pixbuf);
-            }
-
           /* already done */
           needs_state_transform = FALSE;
           break;
@@ -1934,9 +1654,6 @@ gtk_image_expose (GtkWidget      *widget,
                                    image);
               }
 
-            image_bound.width = gdk_pixbuf_animation_get_width (priv->data.anim.anim);
-            image_bound.height = gdk_pixbuf_animation_get_height (priv->data.anim.anim);
-                  
             /* don't advance the anim iter here, or we could get frame changes between two
              * exposes of different areas.
              */
@@ -1963,8 +1680,6 @@ gtk_image_expose (GtkWidget      *widget,
 	  if (pixbuf)
 	    {
 	      g_object_ref (pixbuf);
-	      image_bound.width = gdk_pixbuf_get_width (pixbuf);
-	      image_bound.height = gdk_pixbuf_get_height (pixbuf);
 	    }
 	  break;
 
@@ -1985,107 +1700,54 @@ gtk_image_expose (GtkWidget      *widget,
 	  if (pixbuf)
 	    {
 	      g_object_ref (pixbuf);
-	      image_bound.width = gdk_pixbuf_get_width (pixbuf);
-	      image_bound.height = gdk_pixbuf_get_height (pixbuf);
 	    }
 	  break;
 	  
         case GTK_IMAGE_EMPTY:
+        default:
           g_assert_not_reached ();
+          pixbuf = NULL;
           break;
         }
 
-      if (rectangle_intersect_even (&area, &image_bound))
-        {
-          if (pixbuf)
-            {
-              if (needs_state_transform)
-                {
-                  GtkIconSource *source;
-                  GdkPixbuf *rendered;
-
-                  source = gtk_icon_source_new ();
-                  gtk_icon_source_set_pixbuf (source, pixbuf);
-                  /* The size here is arbitrary; since size isn't
-                   * wildcarded in the souce, it isn't supposed to be
-                   * scaled by the engine function
-                   */
-                  gtk_icon_source_set_size (source,
-                                            GTK_ICON_SIZE_SMALL_TOOLBAR);
-                  gtk_icon_source_set_size_wildcarded (source, FALSE);
-
-                  rendered = gtk_style_render_icon (gtk_widget_get_style (widget),
-                                                    source,
-                                                    gtk_widget_get_direction (widget),
-                                                    gtk_widget_get_state (widget),
-                                                    /* arbitrary */
-                                                    (GtkIconSize)-1,
-                                                    widget,
-                                                    "gtk-image");
-
-                  gtk_icon_source_free (source);
-
-                  g_object_unref (pixbuf);
-                  pixbuf = rendered;
-                }
-
-              if (pixbuf)
-                {
-                  cairo_t *cr = gdk_cairo_create (gtk_widget_get_window (widget));
-                  gdk_cairo_set_source_pixbuf (cr, pixbuf, x, y);
-                  gdk_cairo_rectangle (cr, &image_bound);
-                  cairo_fill (cr);
-                  cairo_destroy (cr);
-                }
-            }
-          else
-            {
-              cairo_t *cr;
-              cairo_pattern_t *mask_pattern;
-
-              switch (priv->storage_type)
-                {
-                case GTK_IMAGE_PIXMAP:
-                  cr = gdk_cairo_create (gtk_widget_get_window (widget));
-
-                  if (mask)
-                    {
-                      /* hack to get the mask pattern */
-                      gdk_cairo_set_source_pixmap (cr, mask, mask_x, mask_y);
-                      mask_pattern = cairo_get_source (cr);
-                      cairo_pattern_reference (mask_pattern);
-
-                      gdk_cairo_set_source_pixmap (cr, priv->data.pixmap.pixmap, x, y);
-                      gdk_cairo_rectangle (cr, &image_bound);
-                      cairo_clip (cr);
-                      cairo_mask (cr, mask_pattern);
-                    }
-                  else 
-                    {
-                      gdk_cairo_set_source_pixmap (cr, priv->data.pixmap.pixmap, x, y);
-                      gdk_cairo_rectangle (cr, &image_bound);
-                      cairo_fill (cr);
-                    }
-
-                  cairo_destroy (cr);
-                  break;
-                case GTK_IMAGE_PIXBUF:
-                case GTK_IMAGE_STOCK:
-                case GTK_IMAGE_ICON_SET:
-                case GTK_IMAGE_ANIMATION:
-		case GTK_IMAGE_ICON_NAME:
-                case GTK_IMAGE_EMPTY:
-		case GTK_IMAGE_GICON:
-                  g_assert_not_reached ();
-                  break;
-                }
-            }
-        } /* if rectangle intersects */      
-
       if (pixbuf)
-	g_object_unref (pixbuf);
+        {
+          if (needs_state_transform)
+            {
+              GtkIconSource *source;
+              GdkPixbuf *rendered;
 
-    } /* if widget is drawable */
+              source = gtk_icon_source_new ();
+              gtk_icon_source_set_pixbuf (source, pixbuf);
+              /* The size here is arbitrary; since size isn't
+               * wildcarded in the souce, it isn't supposed to be
+               * scaled by the engine function
+               */
+              gtk_icon_source_set_size (source,
+                                        GTK_ICON_SIZE_SMALL_TOOLBAR);
+              gtk_icon_source_set_size_wildcarded (source, FALSE);
+              
+              rendered = gtk_style_render_icon (gtk_widget_get_style (widget),
+                                                source,
+                                                gtk_widget_get_direction (widget),
+                                                gtk_widget_get_state (widget),
+                                                /* arbitrary */
+                                                (GtkIconSize)-1,
+                                                widget,
+                                                "gtk-image");
+
+              gtk_icon_source_free (source);
+
+              g_object_unref (pixbuf);
+              pixbuf = rendered;
+            }
+
+          gdk_cairo_set_source_pixbuf (cr, pixbuf, x, y);
+          cairo_paint (cr);
+
+          g_object_unref (pixbuf);
+        }
+    }
 
   return FALSE;
 }
@@ -2100,13 +1762,6 @@ gtk_image_reset (GtkImage *image)
   if (priv->storage_type != GTK_IMAGE_EMPTY)
     g_object_notify (G_OBJECT (image), "storage-type");
 
-  if (priv->mask)
-    {
-      g_object_unref (priv->mask);
-      priv->mask = NULL;
-      g_object_notify (G_OBJECT (image), "mask");
-    }
-
   if (priv->icon_size != DEFAULT_ICON_SIZE)
     {
       priv->icon_size = DEFAULT_ICON_SIZE;
@@ -2115,15 +1770,6 @@ gtk_image_reset (GtkImage *image)
   
   switch (priv->storage_type)
     {
-    case GTK_IMAGE_PIXMAP:
-
-      if (priv->data.pixmap.pixmap)
-        g_object_unref (priv->data.pixmap.pixmap);
-      priv->data.pixmap.pixmap = NULL;
-      
-      g_object_notify (G_OBJECT (image), "pixmap");
-      
-      break;
 
     case GTK_IMAGE_PIXBUF:
 
@@ -2241,10 +1887,9 @@ gtk_image_calc_size (GtkImage *image)
   switch (priv->storage_type)
     {
     case GTK_IMAGE_STOCK:
-      pixbuf = gtk_widget_render_icon (widget,
-				       priv->data.stock.stock_id,
-                                       priv->icon_size,
-                                       NULL);
+      pixbuf = gtk_widget_render_icon_pixbuf (widget,
+                                              priv->data.stock.stock_id,
+                                              priv->icon_size);
       break;
       
     case GTK_IMAGE_ICON_SET:
@@ -2285,19 +1930,35 @@ gtk_image_calc_size (GtkImage *image)
 }
 
 static void
-gtk_image_size_request (GtkWidget      *widget,
-                        GtkRequisition *requisition)
+gtk_image_get_preferred_width (GtkWidget *widget,
+                               gint      *minimum,
+                               gint      *natural)
 {
   GtkImage *image;
   GtkImagePrivate *priv;
-  
+
   image = GTK_IMAGE (widget);
   priv  = image->priv;
 
   gtk_image_calc_size (image);
 
-  requisition->width  = priv->required_width;
-  requisition->height = priv->required_height;
+  *minimum = *natural = priv->required_width;
+}
+
+static void
+gtk_image_get_preferred_height (GtkWidget *widget,
+                                gint      *minimum,
+                                gint      *natural)
+{
+  GtkImage *image;
+  GtkImagePrivate *priv;
+
+  image = GTK_IMAGE (widget);
+  priv  = image->priv;
+
+  gtk_image_calc_size (image);
+
+  *minimum = *natural = priv->required_height;
 }
 
 static void
