@@ -55,6 +55,11 @@
  * To detect changes to the value, you would normally use the
  * #GtkRange::value-changed signal.
  *
+ * Note that using the same upper and lower bounds for the #GtkScale (through
+ * the #GtkRange methods) will hide the slider itself. This is useful for
+ * applications that want to show an undeterminate value on the scale, without
+ * changing the layout of the application (such as movie or music players).
+ *
  * <refsect2 id="GtkScale-BUILDER-UI"><title>GtkScale as GtkBuildable</title>
  * GtkScale supports a custom &lt;marks&gt; element, which
  * can contain multiple &lt;mark&gt; elements. The "value" and "position"
@@ -116,10 +121,13 @@ static void     gtk_scale_get_property            (GObject        *object,
                                                    guint           prop_id,
                                                    GValue         *value,
                                                    GParamSpec     *pspec);
-static void     gtk_scale_size_request            (GtkWidget      *widget,
-                                                   GtkRequisition *requisition);
-static void     gtk_scale_style_set               (GtkWidget      *widget,
-                                                   GtkStyle       *previous);
+static void     gtk_scale_get_preferred_width     (GtkWidget      *widget,
+                                                   gint           *minimum,
+                                                   gint           *natural);
+static void     gtk_scale_get_preferred_height    (GtkWidget      *widget,
+                                                   gint           *minimum,
+                                                   gint           *natural);
+static void     gtk_scale_style_updated           (GtkWidget      *widget);
 static void     gtk_scale_get_range_border        (GtkRange       *range,
                                                    GtkBorder      *border);
 static void     gtk_scale_get_mark_label_size     (GtkScale        *scale,
@@ -195,10 +203,11 @@ gtk_scale_class_init (GtkScaleClass *class)
   gobject_class->get_property = gtk_scale_get_property;
   gobject_class->finalize = gtk_scale_finalize;
 
-  widget_class->style_set = gtk_scale_style_set;
+  widget_class->style_updated = gtk_scale_style_updated;
   widget_class->screen_changed = gtk_scale_screen_changed;
   widget_class->draw = gtk_scale_draw;
-  widget_class->size_request = gtk_scale_size_request;
+  widget_class->get_preferred_width = gtk_scale_get_preferred_width;
+  widget_class->get_preferred_height = gtk_scale_get_preferred_height;
 
   range_class->slider_detail = "Xscale";
   range_class->get_range_border = gtk_scale_get_range_border;
@@ -420,6 +429,7 @@ gtk_scale_init (GtkScale *scale)
 {
   GtkScalePrivate *priv;
   GtkRange *range = GTK_RANGE (scale);
+  GtkStyleContext *context;
 
   scale->priv = G_TYPE_INSTANCE_GET_PRIVATE (scale,
                                              GTK_TYPE_SCALE,
@@ -439,6 +449,9 @@ gtk_scale_init (GtkScale *scale)
   g_signal_connect (scale, "notify::orientation",
                     G_CALLBACK (gtk_scale_orientation_notify),
                     NULL);
+
+  context = gtk_widget_get_style_context (GTK_WIDGET (scale));
+  gtk_style_context_add_class (context, GTK_STYLE_CLASS_SCALE);
 }
 
 static void
@@ -911,8 +924,7 @@ gtk_scale_get_mark_label_size (GtkScale        *scale,
 }
 
 static void
-gtk_scale_style_set (GtkWidget *widget,
-                     GtkStyle  *previous)
+gtk_scale_style_updated (GtkWidget *widget)
 {
   gint slider_length;
   GtkRange *range;
@@ -927,7 +939,7 @@ gtk_scale_style_set (GtkWidget *widget,
 
   _gtk_scale_clear_layout (GTK_SCALE (widget));
 
-  GTK_WIDGET_CLASS (gtk_scale_parent_class)->style_set (widget, previous);
+  GTK_WIDGET_CLASS (gtk_scale_parent_class)->style_updated (widget);
 }
 
 static void
@@ -938,32 +950,54 @@ gtk_scale_screen_changed (GtkWidget *widget,
 }
 
 static void
-gtk_scale_size_request (GtkWidget      *widget,
-                        GtkRequisition *requisition)
+gtk_scale_get_preferred_width (GtkWidget *widget,
+                               gint      *minimum,
+                               gint      *natural)
 {
-  GtkRange *range = GTK_RANGE (widget);
-  gint n1, w1, h1, n2, w2, h2;
-  gint slider_length;
-
-  GTK_WIDGET_CLASS (gtk_scale_parent_class)->size_request (widget, requisition);
+  GTK_WIDGET_CLASS (gtk_scale_parent_class)->get_preferred_width (widget, minimum, natural);
   
-  gtk_widget_style_get (widget, "slider-length", &slider_length, NULL);
-
-
-  if (gtk_orientable_get_orientation (GTK_ORIENTABLE (range)) == GTK_ORIENTATION_HORIZONTAL)
+  if (gtk_orientable_get_orientation (GTK_ORIENTABLE (widget)) == GTK_ORIENTATION_HORIZONTAL)
     {
+      gint n1, w1, h1, n2, w2, h2;
+      gint slider_length;
+      gint w;
+
+      gtk_widget_style_get (widget, "slider-length", &slider_length, NULL);
+
       gtk_scale_get_mark_label_size (GTK_SCALE (widget), GTK_POS_TOP, &n1, &w1, &h1, &n2, &w2, &h2);
 
       w1 = (n1 - 1) * w1 + MAX (w1, slider_length);
       w2 = (n2 - 1) * w2 + MAX (w2, slider_length);
-      requisition->width = MAX (requisition->width, MAX (w1, w2));
+      w = MAX (w1, w2);
+
+      *minimum = MAX (*minimum, w);
+      *natural = MAX (*natural, w);
     }
-  else
+}
+
+static void
+gtk_scale_get_preferred_height (GtkWidget *widget,
+                                gint      *minimum,
+                                gint      *natural)
+{
+  GTK_WIDGET_CLASS (gtk_scale_parent_class)->get_preferred_height (widget, minimum, natural);
+
+
+  if (gtk_orientable_get_orientation (GTK_ORIENTABLE (widget)) == GTK_ORIENTATION_VERTICAL)
     {
+      gint n1, w1, h1, n2, w2, h2;
+      gint slider_length;
+      gint h;
+
+      gtk_widget_style_get (widget, "slider-length", &slider_length, NULL);
+
       gtk_scale_get_mark_label_size (GTK_SCALE (widget), GTK_POS_LEFT, &n1, &w1, &h1, &n2, &w2, &h2);
       h1 = (n1 - 1) * h1 + MAX (h1, slider_length);
       h2 = (n2 - 1) * h1 + MAX (h2, slider_length);
-      requisition->height = MAX (requisition->height, MAX (h1, h2));
+      h = MAX (h1, h2);
+
+      *minimum = MAX (*minimum, h);
+      *natural = MAX (*natural, h);
     }
 }
 
@@ -1000,8 +1034,8 @@ gtk_scale_draw (GtkWidget *widget,
   GtkScale *scale = GTK_SCALE (widget);
   GtkScalePrivate *priv = scale->priv;
   GtkRange *range = GTK_RANGE (scale);
-  GtkStateType state_type;
-  GtkStyle *style;
+  GtkStateFlags state = 0;
+  GtkStyleContext *context;
   gint n_marks;
   gint *marks;
   gint focus_padding;
@@ -1009,7 +1043,7 @@ gtk_scale_draw (GtkWidget *widget,
   gint value_spacing;
   gint min_sep = 4;
 
-  style = gtk_widget_get_style (widget);
+  context = gtk_widget_get_style_context (widget);
   gtk_widget_style_get (widget,
                         "focus-padding", &focus_padding,
                         "slider-width", &slider_width, 
@@ -1021,9 +1055,8 @@ gtk_scale_draw (GtkWidget *widget,
    */
   GTK_WIDGET_CLASS (gtk_scale_parent_class)->draw (widget, cr);
 
-  state_type = GTK_STATE_NORMAL;
   if (!gtk_widget_is_sensitive (widget))
-    state_type = GTK_STATE_INSENSITIVE;
+    state |= GTK_STATE_FLAG_INSENSITIVE;
 
   if (priv->marks)
     {
@@ -1066,8 +1099,12 @@ gtk_scale_draw (GtkWidget *widget,
                   max_pos = find_next_pos (widget, m, marks + i, GTK_POS_TOP, 0) - min_sep;
                 }
 
-              gtk_paint_vline (style, cr, state_type,
-                               widget, "scale-mark", y1, y2, x1);
+              gtk_style_context_save (context);
+              gtk_style_context_add_class (context, GTK_STYLE_CLASS_MARK);
+              gtk_style_context_set_state (context, state);
+
+              gtk_render_line (context, cr,
+                               x1, y1, x1, y2);
 
               if (mark->markup)
                 {
@@ -1092,10 +1129,11 @@ gtk_scale_draw (GtkWidget *widget,
                       min_pos_after = x3 + logical_rect.width + min_sep;
                     }
 
-                  gtk_paint_layout (style, cr, state_type,
-                                    FALSE, widget, "scale-mark",
-                                    x3, y3, layout);
+                  gtk_render_layout (context, cr,
+                                     x3, y3, layout);
                 }
+
+              gtk_style_context_restore (context);
             }
           else
             {
@@ -1115,8 +1153,12 @@ gtk_scale_draw (GtkWidget *widget,
                 }
               y1 = marks[i];
 
-              gtk_paint_hline (style, cr, state_type,
-                               widget, "range-mark", x1, x2, y1);
+              gtk_style_context_save (context);
+              gtk_style_context_add_class (context, GTK_STYLE_CLASS_MARK);
+              gtk_style_context_set_state (context, state);
+
+              gtk_render_line (context, cr,
+                               x1, y1, x2, y1);
 
               if (mark->markup)
                 {
@@ -1141,10 +1183,11 @@ gtk_scale_draw (GtkWidget *widget,
                       min_pos_after = y3 + logical_rect.height + min_sep;
                     }
 
-                  gtk_paint_layout (style, cr, state_type,
-                                    FALSE, widget, "scale-mark",
-                                    x3, y3, layout);
+                  gtk_render_layout (context, cr,
+                                     x3, y3, layout);
                 }
+
+              gtk_style_context_restore (context);
             }
         } 
 
@@ -1165,17 +1208,10 @@ gtk_scale_draw (GtkWidget *widget,
       gtk_scale_get_layout_offsets (scale, &x, &y);
       gtk_widget_get_allocation (widget, &allocation);
 
-      gtk_paint_layout (style,
-                        cr,
-                        state_type,
-			FALSE,
-                        widget,
-                        orientation == GTK_ORIENTATION_HORIZONTAL ?
-                        "hscale" : "vscale",
-                        x - allocation.x,
-                        y - allocation.y,
-                        layout);
-
+      gtk_render_layout (context, cr,
+                         x - allocation.x,
+                         y - allocation.y,
+                         layout);
     }
 
   return FALSE;
