@@ -217,25 +217,36 @@ compute_size_for_orientation (GtkWidget         *widget,
             }
           else
             {
-              int ignored_position = 0;
-              int natural_height;
+              gint ignored_position = 0;
+              gint minimum_height;
+              gint natural_height;
 
 	      /* Pull the base natural height from the cache as it's needed to adjust
 	       * the proposed 'for_size' */
-	      gtk_widget_get_preferred_height (widget, NULL, &natural_height);
+	      gtk_widget_get_preferred_height (widget, &minimum_height, &natural_height);
 
               /* convert for_size to unadjusted height (for_size is a proposed allocation) */
               GTK_WIDGET_GET_CLASS (widget)->adjust_size_allocation (widget,
                                                                      GTK_ORIENTATION_VERTICAL,
-                                                                     &natural_height,
+                                                                     &minimum_height,
+								     &natural_height,
                                                                      &ignored_position,
                                                                      &for_size);
 
 	      push_recursion_check (widget, orientation, for_size);
-              GTK_WIDGET_GET_CLASS (widget)->get_preferred_width_for_height (widget, for_size,
-                                                                              &min_size, &nat_size);
+              GTK_WIDGET_GET_CLASS (widget)->get_preferred_width_for_height (widget, 
+									     MAX (for_size, minimum_height),
+									     &min_size, &nat_size);
 	      pop_recursion_check (widget, orientation);
             }
+#ifndef G_DISABLE_CHECKS
+          if (gtk_widget_get_screen (widget))
+            {
+              guint screen_width = gdk_screen_get_width (gtk_widget_get_screen (widget));
+              min_size = MIN (min_size, screen_width);
+              nat_size = MIN (nat_size, screen_width);
+            }
+#endif
         }
       else
         {
@@ -248,24 +259,35 @@ compute_size_for_orientation (GtkWidget         *widget,
           else
             {
               int ignored_position = 0;
+              int minimum_width;
               int natural_width;
 
 	      /* Pull the base natural width from the cache as it's needed to adjust
 	       * the proposed 'for_size' */
-	      gtk_widget_get_preferred_width (widget, NULL, &natural_width);
+	      gtk_widget_get_preferred_width (widget, &minimum_width, &natural_width);
 
               /* convert for_size to unadjusted width (for_size is a proposed allocation) */
               GTK_WIDGET_GET_CLASS (widget)->adjust_size_allocation (widget,
                                                                      GTK_ORIENTATION_HORIZONTAL,
+								     &minimum_width,
                                                                      &natural_width,
                                                                      &ignored_position,
                                                                      &for_size);
 
 	      push_recursion_check (widget, orientation, for_size);
-              GTK_WIDGET_GET_CLASS (widget)->get_preferred_height_for_width (widget, for_size,
-                                                                              &min_size, &nat_size);
+              GTK_WIDGET_GET_CLASS (widget)->get_preferred_height_for_width (widget, 
+									     MAX (for_size, minimum_width),
+									     &min_size, &nat_size);
 	      pop_recursion_check (widget, orientation);
             }
+#ifndef G_DISABLE_CHECKS
+          if (gtk_widget_get_screen (widget))
+            {
+              guint screen_height = gdk_screen_get_height (gtk_widget_get_screen (widget));
+              min_size = MIN (min_size, screen_height);
+              nat_size = MIN (nat_size, screen_height);
+            }
+#endif
         }
 
       if (min_size > nat_size)
