@@ -510,9 +510,9 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
 
     case GTK_TOOLBAR_BOTH:
       if (text_orientation == GTK_ORIENTATION_HORIZONTAL)
-	box = gtk_vbox_new (FALSE, icon_spacing);
+	box = gtk_box_new (GTK_ORIENTATION_VERTICAL, icon_spacing);
       else
-	box = gtk_hbox_new (FALSE, icon_spacing);
+	box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, icon_spacing);
       if (icon)
 	gtk_box_pack_start (GTK_BOX (box), icon, TRUE, TRUE, 0);
       gtk_box_pack_end (GTK_BOX (box), label, FALSE, TRUE, 0);
@@ -522,7 +522,7 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
     case GTK_TOOLBAR_BOTH_HORIZ:
       if (text_orientation == GTK_ORIENTATION_HORIZONTAL)
 	{
-	  box = gtk_hbox_new (FALSE, icon_spacing);
+	  box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, icon_spacing);
 	  if (icon)
 	    gtk_box_pack_start (GTK_BOX (box), icon, label? FALSE : TRUE, TRUE, 0);
 	  if (label)
@@ -530,7 +530,7 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
 	}
       else
 	{
-	  box = gtk_vbox_new (FALSE, icon_spacing);
+	  box = gtk_box_new (GTK_ORIENTATION_VERTICAL, icon_spacing);
 	  if (icon)
 	    gtk_box_pack_end (GTK_BOX (box), icon, label ? FALSE : TRUE, TRUE, 0);
 	  if (label)
@@ -1269,12 +1269,13 @@ gtk_tool_button_set_label_widget (GtkToolButton *button,
 /**
  * gtk_tool_button_get_label_widget:
  * @button: a #GtkToolButton
- * 
- * Returns the widget used as label on @button. See
- * gtk_tool_button_set_label_widget().
- * 
- * Return value: The widget used as label on @button, or %NULL.
- * 
+ *
+ * Returns the widget used as label on @button.
+ * See gtk_tool_button_set_label_widget().
+ *
+ * Return value: (transfer none): The widget used as label
+ *     on @button, or %NULL.
+ *
  * Since: 2.4
  **/
 GtkWidget *
@@ -1288,12 +1289,13 @@ gtk_tool_button_get_label_widget (GtkToolButton *button)
 /**
  * gtk_tool_button_get_icon_widget:
  * @button: a #GtkToolButton
- * 
- * Return the widget used as icon widget on @button. See
- * gtk_tool_button_set_icon_widget().
- * 
- * Return value: The widget used as icon on @button, or %NULL.
- * 
+ *
+ * Return the widget used as icon widget on @button.
+ * See gtk_tool_button_set_icon_widget().
+ *
+ * Return value: (transfer none): The widget used as icon
+ *     on @button, or %NULL.
+ *
  * Since: 2.4
  **/
 GtkWidget *
